(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.Topologica = factory());
}(this, (function () { 'use strict';

  function Graph () {
    var edges = {};

    var adjacent = function (node) { return edges[node] || []; };

    var addNode = function (node) {
      edges[node] = adjacent(node);
    };

    var addEdge = function (u, v) {
      addNode(u);
      addNode(v);
      adjacent(u).push(v);
    };

    var depthFirstSearch = function (sourceNodes) {
      var visited = {};
      var nodeList = [];

      var DFSVisit = function (node) {
        if (!visited[node]) {
          visit(node);
          nodeList.push(node);
        }
      };

      var visit = function (node) {
        visited[node] = true;
        adjacent(node).forEach(DFSVisit);
      };

      sourceNodes.forEach(visit);

      return nodeList;
    };

    var topologicalSort = function (sourceNodes) { return depthFirstSearch(sourceNodes)
        .reverse(); };

    return {
      addEdge: addEdge,
      topologicalSort: topologicalSort
    };
  }

  var parse = function (dependencies) { return dependencies.split
    ? dependencies.split(',').map(function (str) { return str.trim(); })
    : dependencies; };

  var Topologica = function (options) {
    var values = {};
    var functions = {};
    var graph = Graph();

    var invoke = function (property) {
      functions[property]();
    };

    var set = function (options) {
      graph
        .topologicalSort(Object.keys(options).map(function (property) {
          if (values[property] !== options[property]) {
            values[property] = options[property];
            return property;
          }
        }))
        .forEach(invoke);
    };

    var allDefined = function (dependencies) {
      var arg = {};
      return dependencies.every(function (property) {
        if (values[property] !== undefined){
          arg[property] = values[property];
          return true;
        }
      }) ? arg : null;
    };

    Object.keys(options).forEach(function (property) {
      var fn = options[property];
      var dependencies = parse(fn.dependencies);

      dependencies.forEach(function (input) {
        graph.addEdge(input, property);
      });

      functions[property] = function () {
        var arg = allDefined(dependencies);
        if (arg) {
          values[property] = fn(arg);
        }
      };
    });

    return {
      set: set,
      get: function () { return values; }
    };
  };

  return Topologica;

})));
